#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import sys
import pkg_resources

_HELP_MESSAGE_COMMON = """
         _______  _______  _______  _______  ___      ___           __
        |       ||       ||       ||       ||   |    |___|         |  |         _
        |    ___||   _   ||  _____||  _____||   |     ___          |  |____  __| |__
        |   |___ |  | |  || |_____ | |_____ |   |    |   | _______ |   _   ||__   __|
        |    ___||  |_|  ||_____  ||_____  ||   |___ |   ||   _   ||  | |  |   | |
        |   |    |       | _____| | _____| ||       ||   ||  |_|  ||  | |  |   | |__
        |___|    |_______||_______||_______||_______||___||____   ||__| |__|   |____|
                                                               |  |
                                                           ____|  |
                                                          |_______|
"""


_HELP_MESSAGE_DOWNLOAD = """
    FOSSLight Downloader is a tool to download the package via input URL

    Usage: fosslight_download [option1] <arg1> [options2] <arg2>
     ex) fosslight_download -s http://github.com/fosslight/fosslight -t output_dir -d log_dir

    Required:
        -s\t\t      URL of the package to be downloaded

    Optional:
        -h\t\t      Print help message
        -t\t\t      Output path name
        -d\t\t      Directory name to save the log file"""


class PrintHelpMsg():
    message_suffix = ""

    def __init__(self, value):
        self.message_suffix = value

    def print_help_msg(self, exitopt):
        print(_HELP_MESSAGE_COMMON)
        print(self.message_suffix)

        if exitopt:
            sys.exit()


def print_package_version(pkg_name, msg="", exitopt=True):
    if msg == "":
        msg = f"{pkg_name} Version:"
    cur_version = pkg_resources.get_distribution(pkg_name).version
    print(f'{msg} {cur_version}')

    if exitopt:
        sys.exit(0)


def print_help_msg_download(exitOpt=True):
    helpMsg = PrintHelpMsg(_HELP_MESSAGE_DOWNLOAD)
    helpMsg.print_help_msg(exitOpt)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2022 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import yaml
import logging
import os
import copy
from pathlib import Path
import fosslight_util.constant as constant
from fosslight_util.oss_item import OssItem
from fosslight_util.write_excel import _EMPTY_ITEM_MSG

_logger = logging.getLogger(constant.LOGGER_NAME)


def write_yaml(output_file, sheet_list_origin, separate_yaml=False):
    success = True
    error_msg = ""
    output = ""

    try:
        sheet_list = copy.deepcopy(sheet_list_origin)
        if sheet_list:
            output_files = []
            output_dir = os.path.dirname(output_file)

            Path(output_dir).mkdir(parents=True, exist_ok=True)
            if separate_yaml:
                filename = os.path.splitext(os.path.basename(output_file))[0]
                separate_output_file = os.path.join(output_dir, filename)

            merge_sheet = []
            for sheet_name, sheet_contents in sheet_list.items():
                if sheet_name not in constant.supported_sheet_and_scanner.keys():
                    continue
                if not separate_yaml:
                    merge_sheet.extend(sheet_contents)
                else:
                    output_file = f'{separate_output_file}_{sheet_name}.yaml'
                    convert_sheet_to_yaml(sheet_contents, output_file)
                    output_files.append(output_file)

            if not separate_yaml:
                convert_sheet_to_yaml(merge_sheet, output_file)
                output_files.append(output_file)

            if output_files:
                output = ", ".join(output_files)
        else:
            success = False
            error_msg = _EMPTY_ITEM_MSG
    except Exception as ex:
        error_msg = str(ex)
        success = False

    if not success:
        error_msg = "[Error] Writing yaml:" + error_msg

    return success, error_msg, output


def convert_sheet_to_yaml(sheet_contents, output_file):
    sheet_contents = [list(t) for t in set(tuple(e) for e in sorted(sheet_contents))]

    yaml_dict = {}
    for sheet_item in sheet_contents:
        item = OssItem('')
        item.set_sheet_item(sheet_item)
        create_yaml_with_ossitem(item, yaml_dict)

    with open(output_file, 'w') as f:
        yaml.dump(yaml_dict, f, default_flow_style=False, sort_keys=False)


def create_yaml_with_ossitem(item, yaml_dict):
    item_json = item.get_print_json()
    item_name = item_json.pop("name")

    if item_name not in yaml_dict.keys():
        yaml_dict[item_name] = []
    merged = False
    for oss_info in yaml_dict[item_name]:
        if oss_info.get('version', '') == item.version and \
           oss_info.get('license', []) == item.license and \
           oss_info.get('copyright text', '') == item.copyright and \
           oss_info.get('homepage', '') == item.homepage and \
           oss_info.get('download location', '') == item.download_location and \
           oss_info.get('exclude', False) == item.exclude:
            oss_info.get('source name or path', []).extend(item.source_name_or_path)
            oss_info.pop('comment', None)
            merged = True
            break

    if not merged:
        yaml_dict[item_name].append(item_json)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2023 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import logging
import os
import json
import fosslight_util.constant as constant
from fosslight_util.oss_item import OssItem
from typing import List

logger = logging.getLogger(constant.LOGGER_NAME)
EMPTY_FILE_PATH = '-'


def write_scancodejson(output_dir: str, output_filename: str, oss_list: List[OssItem]):
    json_output = {}
    json_output['headers'] = []
    json_output['summary'] = {}
    json_output['license_detections'] = []
    json_output['files'] = []

    for oi in oss_list:
        if oi.exclude:
            continue
        if not oi.source_name_or_path:
            oi.source_name_or_path = EMPTY_FILE_PATH
        for item_path in oi.source_name_or_path:
            filtered = next(filter(lambda x: x['path'] == item_path, json_output['files']), None)
            if filtered:
                append_oss_item_in_filesitem(oi, filtered)
            else:
                json_output['files'] = add_item_in_files(oi, item_path, json_output['files'])
    with open(os.path.join(output_dir, output_filename), 'w') as f:
        json.dump(json_output, f, sort_keys=False, indent=4)


def append_oss_item_in_filesitem(item, files_item):
    if item.is_binary:
        files_item['is_binary'] = item.is_binary
    if item.name or item.version or item.license or item.copyright or item.download_location or item.comment:
        oss_item = {}
        oss_item['name'] = item.name
        oss_item['version'] = item.version
        oss_item['license'] = item.license
        oss_item['copyright'] = item.copyright
        oss_item['download_location'] = item.download_location
        oss_item['comment'] = item.comment
        files_item['oss'].append(oss_item)
    return files_item


def add_item_in_files(item, item_path, files_list):
    files_item = {}
    files_item['path'] = item_path
    files_item['name'] = os.path.basename(item_path)
    files_item['is_binary'] = item.is_binary
    files_item['base_name'], files_item['extension'] = os.path.splitext(os.path.basename(item_path))
    files_item['oss'] = []
    files_item = append_oss_item_in_filesitem(item, files_item)
    files_list.append(files_item)

    return files_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

LOGGER_NAME = "FOSSLight"

FL_SOURCE = 'FL_Source'
FL_DEPENDENCY = 'FL_Dependency'
FL_BINARY = 'FL_Binary'
supported_sheet_and_scanner = {'SRC': FL_SOURCE,
                               'BIN': FL_BINARY,
                               f'SRC_{FL_SOURCE}': FL_SOURCE,
                               f'SRC_{FL_DEPENDENCY}': FL_DEPENDENCY,
                               f'BIN_{FL_BINARY}': FL_BINARY,
                               f'DEP_{FL_DEPENDENCY}': FL_DEPENDENCY}

# Github : https://github.com/(owner)/(repo)
# npm : https://www.npmjs.com/package/(package)/v/(version)
# npm2 : https://www.npmjs.com/package/@(group)/(package)/v/(version)
# pypi : https://pypi.org/project/(oss_name)/(version)
# pypi2 : https://files.pythonhosted.org/packages/source/(alphabet)/(oss_name)/(oss_name)-(version).tar.gz
# Maven: https://mvnrepository.com/artifact/(group)/(artifact)/(version)
# pub: https://pub.dev/packages/(package)/versions/(version)
# Cocoapods : https://cocoapods.org/(package)
# go : https://pkg.go.dev/(package_name_with_slash)@(version)
PKG_PATTERN = {
    "pypi": r'https?:\/\/pypi\.org\/project\/([^\/]+)[\/]?([^\/]*)',
    "pypi2": r'https?:\/\/files\.pythonhosted\.org\/packages\/source\/[\w]\/([^\/]+)\/[\S]+-([^\-]+)\.tar\.gz',
    "maven": r'https?:\/\/mvnrepository\.com\/artifact\/([^\/]+)\/([^\/]+)\/?([^\/]*)',
    "npm": r'https?:\/\/www\.npmjs\.com\/package\/([^\/\@]+)(?:\/v\/)?([^\/]*)',
    "npm2": r'https?:\/\/www\.npmjs\.com\/package\/(\@[^\/]+\/[^\/]+)(?:\/v\/)?([^\/]*)',
    "pub": r'https?:\/\/pub\.dev\/packages\/([^\/]+)(?:\/versions\/)?([^\/]*)',
    "cocoapods": r'https?:\/\/cocoapods\.org\/pods\/([^\/]+)',
    "go": r'https?:\/\/pkg.go.dev\/([^\@]+)\@?v?([^\/]*)'
}



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2023 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import logging
import os
import copy
import re
from fosslight_util.constant import LOGGER_NAME
from fosslight_util.parsing_yaml import parsing_yml
import fosslight_util.constant as constant
from fosslight_util.oss_item import OssItem

logger = logging.getLogger(LOGGER_NAME)
SBOM_INFO_YAML = r"sbom(-|_)info[\s\S]*.ya?ml"


def correct_with_yaml(correct_filepath, path_to_scan, scanner_oss_list):
    success = True
    msg = ""
    correct_list = {}
    correct_yaml = ""
    if correct_filepath == "":
        correct_filepath = path_to_scan

    path_to_scan = os.path.normpath(path_to_scan)
    correct_filepath = os.path.normpath(correct_filepath)
    for filename in os.listdir(correct_filepath):
        if re.search(SBOM_INFO_YAML, filename, re.IGNORECASE):
            correct_yaml = os.path.join(correct_filepath, filename)
            break
    if not correct_yaml:
        msg = f"Cannot find sbom-info.yaml in {correct_filepath}."
        success = False
        return success, msg, correct_list

    rel_path = os.path.relpath(path_to_scan, correct_filepath)

    yaml_oss_list, _, err_msg = parsing_yml(correct_yaml, os.path.dirname(correct_yaml), print_log=True)

    find_match = False
    matched_yaml = []
    for yitem in yaml_oss_list:
        matched_yaml.append([0]*len(yitem.source_name_or_path))

    for sheet_name, sheet_contents in scanner_oss_list.items():
        if sheet_name not in constant.supported_sheet_and_scanner.keys():
            continue
        correct_contents = copy.deepcopy(sheet_contents)
        for idx, oss_raw_item in enumerate(sheet_contents):
            if len(oss_raw_item) < 9:
                logger.warning(f"sheet list is too short ({len(oss_raw_item)}): {oss_raw_item}")
                continue
            oss_item = OssItem('')
            oss_item.set_sheet_item(oss_raw_item)

            matched_yi = []
            oss_rel_path = os.path.normpath(os.path.join(rel_path, oss_item.source_name_or_path[0]))
            for y_idx, yi in enumerate(yaml_oss_list):
                if not yi.source_name_or_path:
                    continue
                for ys_idx, yi_path in enumerate(yi.source_name_or_path):
                    yi_item = copy.deepcopy(yi)
                    if ((os.path.normpath(yi_path) == os.path.normpath(oss_rel_path))
                       or ((os.path.normpath(oss_rel_path).startswith(os.path.normpath(yi_path.rstrip('*')))))):
                        find_match = True
                        yi_item.source_name_or_path = []
                        yi_item.source_name_or_path = oss_item.source_name_or_path[0]
                        matched_yi.append(yi_item)
                        matched_yaml[y_idx][ys_idx] = 1
            if len(matched_yi) > 0:
                for matched_yi_item in matched_yi:
                    matched_oss_item = copy.deepcopy(matched_yi_item)
                    if matched_oss_item.comment:
                        matched_oss_item.comment += '/'
                    matched_oss_item.comment += 'Loaded from sbom-info.yaml'
                    matched_oss_array = matched_oss_item.get_print_array()[0]
                    correct_contents.append(matched_oss_array)
                oss_item.exclude = True
                if oss_item.comment:
                    oss_item.comment += '/'
                oss_item.comment += 'Excluded by sbom-info.yaml'
                correct_contents[idx] = oss_item.get_print_array()[0]

        if sheet_name == 'SRC_FL_Source':
            for n_idx, ni in enumerate(matched_yaml):
                y_item = copy.deepcopy(yaml_oss_list[n_idx])
                all_matched = False
                if sum(ni) != 0:
                    not_matched_path = []
                    for idx, id in enumerate(ni):
                        if not id:
                            not_matched_path.append(y_item.source_name_or_path[idx])
                    y_item.source_name_or_path = []
                    y_item.source_name_or_path = not_matched_path
                    if len(not_matched_path) == 0:
                        all_matched = True
                if y_item.comment:
                    y_item.comment += '/'
                y_item.comment += 'Added by sbom-info.yaml'
                if not (y_item.source_name_or_path or all_matched):
                    correct_contents.append(y_item.get_print_array()[0])
                    continue
                for y_path in y_item.source_name_or_path:
                    y_item_i = copy.deepcopy(y_item)
                    if not os.path.exists(os.path.normpath(os.path.join(correct_filepath, y_path))):
                        y_item_i.exclude = True
                    y_item_i.source_name_or_path = []
                    y_item_i.source_name_or_path = y_path
                    correct_contents.append(y_item_i.get_print_array()[0])
        correct_list[sheet_name] = correct_contents

    if not find_match:
        success = False
        err_msg = 'No match items in sbom-info.yaml'
        return success, err_msg, yaml_oss_list

    return success, msg, correct_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import os
import sys
import json
import uuid
import logging
from datetime import datetime
from pathlib import Path
import traceback
from typing import Dict, Optional

import fosslight_util.constant as constant


PACKAGE = {
    'requirements.txt': 'pypi',
    'setup.py': 'pypi',
    'package.json': 'npm',
    'pom.xml': 'maven',
    'build.gradle': 'gradle',
    'pubspec.yaml': 'pub',
    'Podfile.lock': 'cocoapods',
    'Package.resolved': 'swift',
    'Cartfile.resolved': 'carthage',
    'go.mod': 'Go'
}

_attributionConfidence = 80
logger = logging.getLogger(constant.LOGGER_NAME)


class AttributionItem():
    def __init__(self,
                 source_name: str,
                 licenseName: str,
                 exclude: bool,
                 copyright: Optional[str] = None,
                 packageName: Optional[str] = None,
                 packageVersion: Optional[str] = None,
                 url: Optional[str] = None,
                 packageType: Optional[str] = None):
        self.attributionConfidence = _attributionConfidence
        self.documentConfidence = _attributionConfidence
        if exclude:
            self.excludeFromNotice = True
        else:
            self.excludeFromNotice = False

        self.source_name = source_name
        if source_name == constant.FL_DEPENDENCY:
            self.preSelected = True
        else:
            self.preSelected = False

        self.licenseName = licenseName
        self.copyright = copyright
        self.packageType = packageType
        self.packageName = packageName
        self.packageVersion = packageVersion
        self.url = url


class Attribution(AttributionItem):
    def __init__(self,
                 source_name: str,
                 licenseName: str,
                 exclude: bool,
                 copyright: Optional[str] = None,
                 packageName: Optional[str] = None,
                 packageVersion: Optional[str] = None,
                 url: Optional[str] = None,
                 packageType: Optional[str] = None):
        super().__init__(source_name, licenseName, exclude, copyright=copyright,
                         packageName=packageName, packageVersion=packageVersion, url=url, packageType=packageType)

        self.uuid = uuid.uuid4()

    def __eq__(self, other):
        if not isinstance(other, Attribution):
            return NotImplemented

        return self.source_name == other.source_name and self.licenseName == other.licenseName and \
            self.copyright == other.copyright and self.packageType == other.packageType and \
            self.packageName == other.packageName and self.packageVersion == other.packageVersion and \
            self.url == other.url

    def get_externalAttribution_dict(self):
        dict = {}
        attributionConfidence = 'attributionConfidence'
        copyright = 'copyright'
        excludeFromNotice = 'excludeFromNotice'
        licenseName = 'licenseName'
        preSelected = 'preSelected'
        source = 'source'
        documentConfidence = 'documentConfidence'
        name = 'name'
        packageName = 'packageName'
        packageType = 'packageType'
        packageVersion = 'packageVersion'
        url = 'url'

        dict[attributionConfidence] = self.attributionConfidence

        dict[source] = {}
        dict[source][documentConfidence] = self.documentConfidence
        dict[source][name] = self.source_name

        dict[excludeFromNotice] = self.excludeFromNotice
        dict[licenseName] = self.licenseName
        dict[preSelected] = self.preSelected

        if self.source_name == constant.FL_SOURCE or constant.FL_BINARY:
            dict[copyright] = self.copyright
            dict[packageName] = self.packageName
            dict[packageVersion] = self.packageVersion
            dict[url] = self.url
        elif self.source_name == constant.FL_DEPENDENCY:
            dict[copyright] = self.copyright
            dict[packageName] = self.packageName
            dict[packageVersion] = self.packageVersion
            dict[url] = self.url
            dict[packageType] = self.packageType

        return {key: value for key, value in dict.items() if value is not None}


def make_metadata():
    success = True
    metadata = {}
    _projectId = 'projectId'
    _fileCreationDate = 'fileCreationDate'

    _start_time = datetime.now().strftime('%Y-%m-%dT%H%M%S')

    metadata[_projectId] = str(0)
    metadata[_fileCreationDate] = _start_time

    return success, metadata


def make_frequentlicenses():
    success = True
    error_msg = ''
    frequentLicenses = {}
    frequentLicensesFile = os.path.join("resources", "frequentLicenselist.json")
    try:
        base_dir = os.path.join(sys._MEIPASS, vars(sys.modules[__name__])['__package__'])
    except Exception:
        base_dir = os.path.dirname(__file__)
    file_withpath = os.path.join(base_dir, frequentLicensesFile)

    try:
        with open(file_withpath, 'r', encoding='utf8') as f:
            frequentLicenses = json.load(f)
    except Exception as e:
        success = False
        error_msg = 'Failed to open and parse ' + str(file_withpath)
        logger.error(error_msg + ": " + str(e))
        logger.error(traceback.format_exc())

    return frequentLicenses, success, error_msg


def write_opossum(filename, sheet_list):
    success = True
    error_msg = ''
    dict = {}
    _metadata_key = 'metadata'
    _resources_key = 'resources'
    _externalAttributions_key = 'externalAttributions'
    _resourcesToAttributions_key = 'resourcesToAttributions'
    _filesWithChildren_key = 'filesWithChildren'
    _attributionBreakpoints_key = 'attributionBreakpoints'

    if sheet_list:
        output_dir = os.path.dirname(filename)
        Path(output_dir).mkdir(parents=True, exist_ok=True)

        success, dict[_metadata_key] = make_metadata()
        resources = {}
        fc_list = []
        dict[_resources_key] = {}
        dict[_externalAttributions_key] = {}
        dict[_resourcesToAttributions_key] = {}
        filesWithChildren_list = []
        attributionBreakpoints_list = []
        try:
            for sheet_name, sheet_contents in sheet_list.items():
                if sheet_name in constant.supported_sheet_and_scanner.keys():
                    scanner = constant.supported_sheet_and_scanner.get(sheet_name)
                else:
                    logger.warning("Not supported scanner(sheet_name):" + sheet_name)
                    continue

                ret_resources_attribution = make_resources_and_attributions(sheet_contents, scanner, resources, fc_list)
                success, rsc, ea, ra, fl, ab = ret_resources_attribution
                if success:
                    dict[_resources_key].update(rsc)
                    dict[_externalAttributions_key].update(ea)
                    dict[_resourcesToAttributions_key].update(ra)
                    filesWithChildren_list.extend(fl)
                    attributionBreakpoints_list.extend(ab)

            if success:
                dict[_filesWithChildren_key] = filesWithChildren_list
                dict[_attributionBreakpoints_key] = attributionBreakpoints_list
                frequentLicenses, success, error_msg = make_frequentlicenses()
                dict.update(frequentLicenses)

                opossum_json = json.dumps(dict)
                with open(filename, 'w+') as json_f:
                    json_f.write(opossum_json)
        except Exception as e:
            error_msg = 'Failed to write opossum json' + str(e)
            logger.error(traceback.format_exc())
    else:
        success = False
        error_msg = 'No item to write in output file.'

    return success, error_msg


def make_resources(path, resources):
    files = os.path.basename(path)
    dirs = os.path.dirname(path).split(os.sep)

    try:
        if not dirs[0]:
            resources[files] = 1
        else:
            tmp = resources
            for dir in dirs:
                tmp = tmp.setdefault(dir, {})
                if dir == dirs[-1]:
                    tmp[files] = 1
    except Exception as e:
        logger.error("Failed to make resources: " + str(e) + traceback.format_exc())

    return resources


def make_resources_and_attributions(sheet_items, scanner, resources, fc_list):
    success = True
    resourcesToAttributions = {}
    externalAttributions: Dict[str, Attribution] = {}
    externalAttribution_list = []
    ab_list = []

    try:
        for items in sheet_items:
            items = items[0:9]
            path, oss_name, oss_version, license, url, homepage, copyright, exclude, comment = items

            if scanner == constant.FL_SOURCE:
                if (os.path.join(os.sep, path) + os.sep) not in fc_list:
                    resources = make_resources(path, resources)
                attribution = Attribution(scanner, license, exclude, copyright, oss_name, oss_version, url)
            elif scanner == constant.FL_BINARY:
                resources = make_resources(path, resources)
                attribution = Attribution(scanner, license, exclude, copyright, oss_name, oss_version, url)
            elif scanner == constant.FL_DEPENDENCY:
                try:
                    packageType = PACKAGE[path]
                except Exception:
                    packageType = ''
                    if path == '':
                        try_package_type = oss_name.split(':')[0]
                    else:
                        try_package_type = path.split(',')[0]
                    if try_package_type in PACKAGE.items():
                        packageType = try_package_type

                if (os.path.join(os.sep, path) + os.sep) not in fc_list:
                    fc_list.append(os.path.join(os.sep, path) + os.sep)
                    ab_list.append(os.path.join(os.sep, path, packageType) + os.sep)
                    if path in resources.keys():
                        del resources[path]
                path = os.path.join(path, packageType, oss_name)
                resources = make_resources(path, resources)
                attribution = Attribution(scanner, license, exclude,
                                          copyright, oss_name, oss_version, url, packageType)
            else:
                logger.error("Not supported scanner:" + scanner)
                break

            find_same_attribution = False
            uuid = ''
            for externalAttribution in externalAttributions.values():
                if attribution == externalAttribution:
                    find_same_attribution = True
                    uuid = externalAttribution.uuid
                    del attribution
                    break

            if not find_same_attribution:
                externalAttribution_list.append(attribution)
                uuid = attribution.uuid

            uuid_list = []
            uuid_list.append(str(uuid))
            if os.path.join(os.sep, path) in resourcesToAttributions:
                resourcesToAttributions[os.path.join(os.sep, path)].extend(uuid_list)
            else:
                resourcesToAttributions[os.path.join(os.sep, path)] = uuid_list

            for ext in externalAttribution_list:
                externalAttributions[str(ext.uuid)] = ext
    except Exception as e:
        logger.error("Failed to make_resources_and_attributions: " + str(e))
        logger.error(traceback.format_exc())
        success = False

    externalAttributionsConverted = {
        uuid: ext.get_externalAttribution_dict() for uuid, ext in externalAttributions.items()
    }

    return success, resources, externalAttributionsConverted, resourcesToAttributions, fc_list, ab_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2022 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import os
import sys
import logging
import yaml
import re
from fosslight_util.constant import LOGGER_NAME
from fosslight_util.write_yaml import create_yaml_with_ossitem
from fosslight_util.read_excel import read_oss_report

logger = logging.getLogger(LOGGER_NAME)


def find_report_file(path_to_find):
    file_to_find = ["FOSSLight-Report.xlsx", "OSS-Report.xlsx"]

    try:
        for file in file_to_find:
            file_with_path = os.path.join(path_to_find, file)
            if os.path.isfile(file_with_path):
                return file
        for root, dirs, files in os.walk(path_to_find):
            for file in files:
                file_name = file.lower()
                p = re.compile(r"[\s\S]*OSS[\s\S]*-Report[\s\S]*.xlsx", re.I)
                if p.search(file_name):
                    return os.path.join(root, file)
    except Exception as error:
        logger.debug("Find report:"+str(error))
    return ""


def convert_excel_to_yaml(oss_report_to_read, output_file, sheet_names=""):
    _file_extension = ".yaml"
    yaml_dict = {}

    if os.path.isfile(oss_report_to_read):
        try:
            items = read_oss_report(oss_report_to_read, sheet_names)
            for item in items:
                create_yaml_with_ossitem(item, yaml_dict)
            if yaml_dict:
                output_file = output_file if output_file.endswith(_file_extension) else output_file + _file_extension
                success = write_yaml_file(output_file, yaml_dict)
                if success:
                    logger.warning(f"Output: {output_file}")
                else:
                    logger.error(f"Can't write yaml file : {output_file}")
                    sys.exit(1)
        except Exception as error:
            logger.error(f"Convert yaml: {error}")
    else:
        logger.error(f"Can't find a file: {oss_report_to_read}")


def write_yaml_file(output_file, json_output):
    success = True
    error_msg = ""

    try:
        with open(output_file, 'w') as f:
            yaml.dump(json_output, f, sort_keys=False)
    except Exception as ex:
        error_msg = str(ex)
        success = False
    return success, error_msg



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import logging
import os
from fosslight_util.constant import LOGGER_NAME

_logger = logging.getLogger(LOGGER_NAME)


class OssItem:
    def __init__(self, value):
        self._name = ""
        self._version = ""
        self._license = []
        self._copyright = ""
        self.comment = ""
        self._exclude = False
        self.homepage = ""
        self.relative_path = value
        self._source_name_or_path = []
        self.download_location = ""
        self._yocto_recipe = []
        self._yocto_package = []
        self.is_binary = False

    def __del__(self):
        pass

    @property
    def copyright(self):
        return self._copyright

    @copyright.setter
    def copyright(self, value):
        if value != "":
            if isinstance(value, list):
                value = "\n".join(value)
            value = value.strip()
        self._copyright = value

    @property
    def exclude(self):
        return self._exclude

    @exclude.setter
    def exclude(self, value):
        if value:
            self._exclude = True
        else:
            self._exclude = False

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        if value != "":
            self._name = value

    @property
    def version(self):
        return self._version

    @version.setter
    def version(self, value):
        if value:
            self._version = str(value)
        else:
            self._version = ""

    @property
    def license(self):
        return self._license

    @license.setter
    def license(self, value):
        if not isinstance(value, list):
            value = value.split(",")
        self._license.extend(value)
        self._license = [item.strip() for item in self._license]
        self._license = list(set(self._license))

    @property
    def source_name_or_path(self):
        return self._source_name_or_path

    @source_name_or_path.setter
    def source_name_or_path(self, value):
        if not value:
            self._source_name_or_path = []
        else:
            if not isinstance(value, list):
                value = value.split(",")
            self._source_name_or_path.extend(value)
            self._source_name_or_path = [item.strip() for item in self._source_name_or_path]
            self._source_name_or_path = list(set(self._source_name_or_path))

    @property
    def yocto_recipe(self):
        return self._yocto_recipe

    @yocto_recipe.setter
    def yocto_recipe(self, value):
        if not isinstance(value, list):
            value = value.split(",")
        self._yocto_recipe.extend(value)
        self._yocto_recipe = [item.strip() for item in self._yocto_recipe]
        self._yocto_recipe = list(set(self._yocto_recipe))

    @property
    def yocto_package(self):
        return self._yocto_package

    @yocto_package.setter
    def yocto_package(self, value):
        if not isinstance(value, list):
            value = value.split(",")
        self._yocto_package.extend(value)
        self._yocto_package = [item.strip() for item in self._yocto_package]
        self._yocto_package = list(set(self._yocto_package))

    def set_sheet_item(self, item):
        if len(item) < 9:
            _logger.warning(f"sheet list is too short ({len(item)}): {item}")
            return
        self.source_name_or_path = item[0]
        self.name = item[1]
        self.version = item[2]
        self.license = item[3]
        self.download_location = item[4]
        self.homepage = item[5]
        self.copyright = item[6]
        self.exclude = item[7]
        self.comment = item[8]

    def get_print_array(self):
        items = []
        if len(self.source_name_or_path) == 0:
            self.source_name_or_path.append("")
        if len(self.license) == 0:
            self.license.append("")

        exclude = "Exclude" if self.exclude else ""

        for source_name_or_path in self.source_name_or_path:
            lic = ",".join(self.license)
            items.append([os.path.join(self.relative_path, source_name_or_path), self.name, self.version, lic,
                          self.download_location, self.homepage, self.copyright, exclude, self.comment])
        return items

    def get_print_json(self):
        json_item = {}
        json_item["name"] = self.name

        json_item["version"] = self.version
        if len(self.source_name_or_path) > 0:
            json_item["source name or path"] = self.source_name_or_path
        if len(self.license) > 0:
            json_item["license"] = self.license
        if self.download_location != "":
            json_item["download location"] = self.download_location
        if self.homepage != "":
            json_item["homepage"] = self.homepage
        if self.copyright != "":
            json_item["copyright text"] = self.copyright
        if self.exclude:
            json_item["exclude"] = self.exclude
        if self.comment != "":
            json_item["comment"] = self.comment

        return json_item


def invalid(cmd):
    _logger.info('[{}] is invalid'.format(cmd))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import logging
from typing import List, Dict, Any
import xlrd
import json
from fosslight_util.constant import LOGGER_NAME
from fosslight_util.oss_item import OssItem
from fosslight_util.parsing_yaml import set_value_switch

logger = logging.getLogger(LOGGER_NAME)
IDX_CANNOT_FOUND = -1
PREFIX_BIN = "bin"
SHEET_PREFIX_TO_READ = ["bin", "bom", "src"]
xlrd.xlsx.ensure_elementtree_imported(False, None)
xlrd.xlsx.Element_has_iter = True


def read_oss_report(excel_file: str, sheet_names: str = "") -> List[OssItem]:
    oss_report_items: List[OssItem] = []
    xl_sheets: Dict[str, Any] = {}
    all_sheet_to_read: List[str] = []
    not_matched_sheet: List[str] = []
    any_sheet_matched = False

    if sheet_names:
        sheet_name_prefix_match = False
        sheet_name_to_read = sheet_names.split(",")
    else:
        sheet_name_prefix_match = True
        sheet_name_to_read = SHEET_PREFIX_TO_READ

    try:
        logger.info(f"Read data from : {excel_file}")
        xl_workbook = xlrd.open_workbook(excel_file)
        all_sheet_in_excel = xl_workbook.sheet_names()

        for sheet_to_read in sheet_name_to_read:
            try:
                any_sheet_matched = False
                sheet_to_read_lower = sheet_to_read.lower()
                all_sheet_to_read.append(sheet_to_read_lower)
                for sheet_name in all_sheet_in_excel:
                    sheet_name_lower = sheet_name.lower()
                    if (sheet_name_prefix_match and sheet_name_lower.startswith(sheet_to_read_lower)) \
                       or sheet_to_read_lower == sheet_name_lower:
                        sheet = xl_workbook.sheet_by_name(sheet_name)
                        if sheet:
                            xl_sheets[sheet_name] = sheet
                            any_sheet_matched = True
                if not any_sheet_matched:
                    not_matched_sheet.append(sheet_to_read)
            except Exception as error:
                logger.debug(f"Failed to load sheet: {sheet_name} {error}")

        # Not matched any sheet
        if len(sheet_name_to_read) == len(not_matched_sheet):
            logger.warning("No sheet names are matched.")
        # Partially matched
        elif (not sheet_name_prefix_match) and not_matched_sheet:
            logger.warning(f"Not matched sheet name: {not_matched_sheet}")

        for sheet_name, xl_sheet in xl_sheets.items():
            _item_idx = {
                "ID": IDX_CANNOT_FOUND,
                "Source Name or Path": IDX_CANNOT_FOUND,
                "Binary Name": IDX_CANNOT_FOUND,
                "OSS Name": IDX_CANNOT_FOUND,
                "OSS Version": IDX_CANNOT_FOUND,
                "License": IDX_CANNOT_FOUND,
                "Download Location": IDX_CANNOT_FOUND,
                "Homepage": IDX_CANNOT_FOUND,
                "Exclude": IDX_CANNOT_FOUND,
                "Copyright Text": IDX_CANNOT_FOUND,
                "Comment": IDX_CANNOT_FOUND,
                "File Name or Path": IDX_CANNOT_FOUND
            }
            num_cols = xl_sheet.ncols
            num_rows = xl_sheet.nrows
            MAX_FIND_HEADER_COLUMN = 5 if num_rows > 5 else num_rows
            DATA_START_ROW_IDX = 1
            for row_idx in range(0, MAX_FIND_HEADER_COLUMN):
                for col_idx in range(row_idx, num_cols):
                    cell_obj = xl_sheet.cell(row_idx, col_idx)
                    if cell_obj.value in _item_idx:
                        _item_idx[cell_obj.value] = col_idx

                if len([key for key, value in _item_idx.items() if value != IDX_CANNOT_FOUND]) > 3:
                    DATA_START_ROW_IDX = row_idx + 1
                    break

            # Get all values, iterating through rows and columns
            column_keys = json.loads(json.dumps(_item_idx))

            is_bin = True if sheet_name.lower().startswith(PREFIX_BIN) else False

            for row_idx in range(DATA_START_ROW_IDX, xl_sheet.nrows):
                item = OssItem("")
                item.is_binary = is_bin
                valid_row = True
                load_data_cnt = 0

                for column_key, column_idx in column_keys.items():
                    if column_idx != IDX_CANNOT_FOUND:
                        cell_obj = xl_sheet.cell(row_idx, column_idx)
                        cell_value = cell_obj.value
                        if cell_value != "":
                            if column_key != "ID":
                                if column_key:
                                    column_key = column_key.lower().strip()
                                set_value_switch(item, column_key, cell_value)
                                load_data_cnt += 1
                            else:
                                valid_row = False if cell_value == "-" else True
                if valid_row and load_data_cnt > 0:
                    oss_report_items.append(item)

    except Exception as error:
        logger.error(f"Parsing a OSS Report: {error}")
    return oss_report_items



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import xlsxwriter
import csv
import time
import logging
import os
import platform
import pandas as pd
import copy
from pathlib import Path
import fosslight_util.constant as constant
from jsonmerge import merge

_HEADER = {'BIN (': ['ID', 'Binary Name', 'Source Code Path',
                     'NOTICE.html', 'OSS Name', 'OSS Version',
                     'License', 'Download Location', 'Homepage',
                     'Copyright Text', 'Exclude', 'Comment'],
           'SRC': ['ID', 'Source Name or Path', 'OSS Name',
                   'OSS Version', 'License',  'Download Location',
                   'Homepage', 'Copyright Text', 'Exclude',
                   'Comment'],
           'BIN': ['ID', 'Binary Name', 'OSS Name', 'OSS Version',
                   'License', 'Download Location', 'Homepage',
                   'Copyright Text', 'Exclude', 'Comment']}
_OUTPUT_FILE_PREFIX = "FOSSLight-Report_"
_EMPTY_ITEM_MSG = "* There is no item"\
                    " to print in FOSSLight-Report.\n"
IDX_FILE = 0
IDX_EXCLUDE = 7
logger = logging.getLogger(constant.LOGGER_NAME)


def write_excel_and_csv(filename_without_extension, sheet_list, ignore_os=False, extended_header={}, hide_header={}):
    success = True
    error_msg = ""
    success_csv = True
    error_msg_csv = ""
    output_files = ""
    output_csv = ""

    is_not_null, sheet_list = remove_empty_sheet(sheet_list)

    if is_not_null:
        output_dir = os.path.dirname(filename_without_extension)
        Path(output_dir).mkdir(parents=True, exist_ok=True)

        success, error_msg = write_result_to_excel(f"{filename_without_extension}.xlsx",
                                                   sheet_list,
                                                   extended_header,
                                                   hide_header)

        if ignore_os or platform.system() != "Windows":
            success_csv, error_msg_csv, output_csv = write_result_to_csv(f"{filename_without_extension}.csv",
                                                                         sheet_list, True, extended_header)
        if success:
            output_files = f"{filename_without_extension}.xlsx"
        else:
            error_msg = "[Error] Writing excel:" + error_msg
        if success_csv:
            if output_csv:
                output_files = f"{output_files}, {output_csv}" if output_files else output_csv
        else:
            error_msg += "\n[Error] Writing csv:" + error_msg_csv
    else:
        success = False
        error_msg = _EMPTY_ITEM_MSG

    return (success and success_csv), error_msg, output_files


def remove_empty_sheet(sheet_items):
    skip_sheet_name = []
    cnt_sheet_to_print = 0
    final_sheet_to_print = {}
    success = False
    try:
        if sheet_items:
            for sheet_name, sheet_content in sheet_items.items():
                if len(sheet_content) > 0:
                    final_sheet_to_print[sheet_name] = sheet_content
                    cnt_sheet_to_print += 1
                else:
                    skip_sheet_name.append(sheet_name)
            if cnt_sheet_to_print != 0:
                success = True
                if len(skip_sheet_name) > 0:
                    logger.warn("* Empty sheet(not printed):" + str(skip_sheet_name))
    except Exception as ex:
        logger.warn("* Warning:"+str(ex))

    return success, final_sheet_to_print


def get_header_row(sheet_name, sheet_content, extended_header={}):
    selected_header = []

    merged_headers = merge(_HEADER, extended_header)

    selected_header = merged_headers.get(sheet_name)
    if not selected_header:
        for header_key in merged_headers.keys():
            if sheet_name.startswith(header_key):
                selected_header = merged_headers[header_key]
                break

    if not selected_header:
        selected_header = sheet_content.pop(0)
    return selected_header, sheet_content


def write_result_to_csv(output_file, sheet_list_origin, separate_sheet=False, extended_header={}):
    success = True
    error_msg = ""
    file_extension = ".csv"
    output = ""

    try:
        sheet_list = copy.deepcopy(sheet_list_origin)
        if sheet_list:
            output_files = []
            output_dir = os.path.dirname(output_file)
            Path(output_dir).mkdir(parents=True, exist_ok=True)
            if separate_sheet:
                filename = os.path.splitext(os.path.basename(output_file))[0]
                separate_output_file = os.path.join(output_dir, filename)

            merge_sheet = []
            for sheet_name, sheet_contents in sheet_list.items():
                row_num = 1
                header_row, sheet_content_without_header = get_header_row(sheet_name, sheet_contents[:], extended_header)

                if 'Copyright Text' in header_row:
                    idx = header_row.index('Copyright Text')-1
                    for item in sheet_content_without_header:
                        item[idx] = item[idx].replace('\n', ', ')
                if not separate_sheet:
                    merge_sheet.extend(sheet_content_without_header)
                    if sheet_name == list(sheet_list.keys())[-1]:
                        sheet_content_without_header = merge_sheet
                    else:
                        continue
                else:
                    output_file = separate_output_file + "_" + sheet_name + file_extension
                try:
                    sheet_content_without_header = sorted(sheet_content_without_header,
                                                          key=lambda x: (x[IDX_EXCLUDE], x[IDX_FILE] == "", x[IDX_FILE]))
                except Exception:
                    pass
                with open(output_file, 'w', newline='') as file:
                    writer = csv.writer(file, delimiter='\t')
                    writer.writerow(header_row)
                    for row_item in sheet_content_without_header:
                        row_item.insert(0, row_num)
                        writer.writerow(row_item)
                        row_num += 1
                output_files.append(output_file)
            if output_files:
                output = ", ".join(output_files)
    except Exception as ex:
        error_msg = str(ex)
        success = False

    return success, error_msg, output


def write_result_to_excel(out_file_name, sheet_list, extended_header={}, hide_header={}):
    success = True
    error_msg = ""

    try:
        if sheet_list:
            output_dir = os.path.dirname(out_file_name)
            Path(output_dir).mkdir(parents=True, exist_ok=True)

            workbook = xlsxwriter.Workbook(out_file_name)
            for sheet_name, sheet_contents in sheet_list.items():
                selected_header, sheet_content_without_header = get_header_row(sheet_name, sheet_contents[:], extended_header)
                try:
                    sheet_content_without_header = sorted(sheet_content_without_header,
                                                          key=lambda x: (x[IDX_EXCLUDE], x[IDX_FILE] == "", x[IDX_FILE]))
                except Exception:
                    pass
                worksheet = create_worksheet(workbook, sheet_name, selected_header)
                write_result_to_sheet(worksheet, sheet_content_without_header)

                if hide_header:
                    hide_column(worksheet, selected_header, hide_header)
            workbook.close()
    except Exception as ex:
        error_msg = str(ex)
        success = False
    return success, error_msg


def write_result_to_sheet(worksheet, sheet_contents):
    row = 1
    for row_item in sheet_contents:
        worksheet.write(row, 0, row)
        for col_num, value in enumerate(row_item):
            worksheet.write(row, col_num + 1, str(value))
        row += 1


def hide_column(worksheet, selected_header, hide_header):
    for col_idx, sel_hd in enumerate(selected_header):
        for hide_hd in hide_header:
            if str(sel_hd).lower() == str(hide_hd).lower():
                worksheet.set_column(col_idx, col_idx, None, None, {"hidden": True})


def create_worksheet(workbook, sheet_name, header_row):
    if len(sheet_name) > 31:
        current_time = str(time.time())
        sheet_name = current_time
    worksheet = workbook.add_worksheet(sheet_name)
    for col_num, value in enumerate(header_row):
        worksheet.write(0, col_num, value)
    return worksheet


def merge_excels(find_excel_dir, final_out, merge_files=''):
    success = True
    msg = ""
    FIND_EXTENSION = '.xlsx'
    added_sheet_names = []
    try:
        files = os.listdir(find_excel_dir)

        if len([name for name in files if name.endswith(FIND_EXTENSION)]) > 0:
            writer = pd.ExcelWriter(final_out)

            for file in files:
                if merge_files:
                    if file not in merge_files:
                        continue
                if file.endswith(FIND_EXTENSION):
                    f_short_name = os.path.splitext(
                        file)[0].replace(_OUTPUT_FILE_PREFIX, "")
                    file = os.path.join(find_excel_dir, file)
                    excel_file = pd.ExcelFile(file, engine='openpyxl')

                    for sheet_name in excel_file.sheet_names:
                        sheet_name_to_copy = f"{f_short_name}_{sheet_name}"
                        df_excel = pd.read_excel(
                            file, sheet_name=sheet_name, engine='openpyxl')
                        if sheet_name not in added_sheet_names:
                            sheet_name_to_copy = sheet_name
                        df_excel.to_excel(writer, sheet_name_to_copy,
                                          index=False)
            writer.close()
    except Exception as ex:
        msg = str(ex)
        success = False

    return success, msg



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

from urllib.request import urlopen
import os
import sys
import json
import traceback

_resources_dir = 'resources'
_licenses_json_file = 'licenses.json'
_frequentLicenselist_file = 'frequentLicenselist.json'
_frequent_lic_nick_json_file = 'frequent_license_nick_list.json'


def get_license_from_nick():
    licenses = {}
    licenses_file = os.path.join(_resources_dir, _frequent_lic_nick_json_file)

    try:
        base_dir = os.path.join(sys._MEIPASS, vars(sys.modules[__name__])['__package__'])
    except Exception:
        base_dir = os.path.dirname(__file__)

    file_withpath = os.path.join(base_dir, licenses_file)
    try:
        with open(file_withpath, 'r') as f:
            licenses = json.load(f)
    except Exception as ex:
        print(f"Error to get license from json file : {ex}")

    return licenses


def get_spdx_licenses_json():
    success = True
    error_msg = ''
    licenses = ''
    # licenses : https://github.com/spdx/license-list-data/blob/v3.17/json/licenses.json
    licenses_file = os.path.join(_resources_dir, _licenses_json_file)

    try:
        base_dir = os.path.join(sys._MEIPASS, vars(sys.modules[__name__])['__package__'])
    except Exception:
        base_dir = os.path.dirname(__file__)

    file_withpath = os.path.join(base_dir, licenses_file)
    try:
        with open(file_withpath, 'r') as f:
            licenses = json.load(f)
    except Exception as e:
        success = False
        error_msg = 'Failed to open ' + file_withpath + ': ' + str(e)

    return success, error_msg, licenses


def create_frequentlicenses():
    success = True
    error_msg = ''
    licenses = ''
    frequentLicenses = {}
    _frequentLicenses_key = 'frequentLicenses'

    # spdx_txt_url = _spdx_txt_base_url + version + /text/ + licenseId + '.txt'
    _spdx_txt_base_url = 'https://raw.githubusercontent.com/spdx/license-list-data/'

    try:
        success, error_msg, licenses = get_spdx_licenses_json()
        if success:
            version = licenses['licenseListVersion']
            frequentLicenses[_frequentLicenses_key] = []
            for lic in licenses['licenses']:
                tmp_lic = {}
                tmp_lic["fullName"] = lic['name']
                tmp_lic["shortName"] = lic['licenseId']

                deprecated = lic['isDeprecatedLicenseId']
                if deprecated:
                    spdx_txt_url = _spdx_txt_base_url + 'v' + version + '/text/' + 'deprecated_' + lic['licenseId'] + '.txt'
                else:
                    spdx_txt_url = _spdx_txt_base_url + 'v' + version + '/text/' + lic['licenseId'] + '.txt'

                with urlopen(spdx_txt_url) as url:
                    licenseText = url.read().decode('utf-8')
                tmp_lic["defaultText"] = licenseText

                frequentLicenses[_frequentLicenses_key].append(tmp_lic)

    except Exception as e:
        success = False
        error_msg = 'Failed to open and parse licenses.json'
        print(error_msg, ": ", e)
        print(traceback.format_exc())

    return frequentLicenses, success, error_msg


def main():
    _frequencyLicense = os.path.join(_resources_dir, _frequentLicenselist_file)
    frequentLicenses, success, error_msg = create_frequentlicenses()
    if success:
        with open(_frequencyLicense, 'w', encoding='utf-8') as f:
            json.dump(frequentLicenses, f, sort_keys=True, indent=4)
        print("Created " + _frequentLicenselist_file + " in " + _resources_dir)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2020 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import os
import sys
import wget
import tarfile
import zipfile
import logging
import argparse
import shutil
import pygit2 as git
import bz2
import contextlib
from datetime import datetime
from pathlib import Path
from fosslight_util._get_downloadable_url import get_downloadable_url
from fosslight_util.help import print_help_msg_download
import fosslight_util.constant as constant
from fosslight_util.set_log import init_log
import signal
import time
import threading
import platform
import subprocess
import re
from typing import Tuple

logger = logging.getLogger(constant.LOGGER_NAME)
compression_extension = {".tar.bz2", ".tar.gz", ".tar.xz", ".tgz", ".tar", ".zip", ".jar", ".bz2"}
prefix_refs = ["refs/remotes/origin/", "refs/tags/"]
SIGNAL_TIMEOUT = 600


class Alarm(threading.Thread):
    def __init__(self, timeout):
        threading.Thread.__init__(self)
        self.timeout = timeout
        self.setDaemon(True)

    def run(self):
        time.sleep(self.timeout)
        logger.error("download timeout! (%d sec)", SIGNAL_TIMEOUT)
        os._exit(1)


class TimeOutException(Exception):
    pass


def alarm_handler(signum, frame):
    logger.warning("download timeout! (%d sec)", SIGNAL_TIMEOUT)
    raise TimeOutException()


def change_src_link_to_https(src_link):
    src_link = src_link.replace("git://", "https://")
    if src_link.endswith(".git"):
        src_link = src_link.replace('.git', '')
    return src_link


def parse_src_link(src_link):
    src_info = {}
    src_link_changed = ""
    if src_link.startswith("git://") or src_link.startswith("https://") or src_link.startswith("http://"):
        src_link_split = src_link.split(';')
        if src_link.startswith("git://github.com/"):
            src_link_changed = change_src_link_to_https(src_link_split[0])
        else:
            if "rubygems.org" in src_link:
                src_info["rubygems"] = True
            src_link_changed = src_link_split[0]

        branch_info = [s for s in src_link_split if s.startswith('branch')]
        tag_info = [s for s in src_link_split if s.startswith('tag')]

        src_info["url"] = src_link_changed
        src_info["branch"] = branch_info
        src_info["tag"] = tag_info
        return src_info


def main():
    parser = argparse.ArgumentParser(description='FOSSLight Downloader', prog='fosslight_download', add_help=False)
    parser.add_argument('-h', '--help', help='Print help message', action='store_true', dest='help')
    parser.add_argument('-s', '--source', help='Source link to download', type=str, dest='source')
    parser.add_argument('-t', '--target_dir', help='Target directory', type=str, dest='target_dir', default="")
    parser.add_argument('-d', '--log_dir', help='Directory to save log file', type=str, dest='log_dir', default="")

    src_link = ""
    target_dir = os.getcwd()
    log_dir = os.getcwd()

    try:
        args = parser.parse_args()
    except SystemExit:
        sys.exit(0)

    if args.help:
        print_help_msg_download()
    if args.source:
        src_link = args.source
    if args.target_dir:
        target_dir = args.target_dir
    if args.log_dir:
        log_dir = args.log_dir

    if not src_link:
        print_help_msg_download()
    else:
        cli_download_and_extract(src_link, target_dir, log_dir)


def cli_download_and_extract(link: str, target_dir: str, log_dir: str, checkout_to: str = "",
                             compressed_only: bool = False) -> Tuple[bool, str, str, str]:
    global logger

    success = True
    msg = ""
    oss_name = ""
    oss_version = ""
    log_file_name = "fosslight_download_" + \
        datetime.now().strftime('%Y%m%d_%H-%M-%S')+".txt"
    logger, log_item = init_log(os.path.join(log_dir, log_file_name))
    link = link.strip()

    try:
        if not link:
            success = False
            msg = "Need a link to download."
        elif os.path.isfile(target_dir):
            success = False
            msg = f"The target directory exists as a file.: {target_dir}"
        else:
            src_info = parse_src_link(link)
            link = src_info.get("url", "")
            tag = ''.join(src_info.get("tag",  "")).split('=')[-1]
            branch = ''.join(src_info.get("branch", "")).split('=')[-1]
            is_rubygems = src_info.get("rubygems", False)

            # General download (git clone, wget)
            success_git, msg, oss_name, oss_version = download_git_clone(link, target_dir, checkout_to, tag, branch)
            if (not is_rubygems) and (not success_git):
                if os.path.isfile(target_dir):
                    shutil.rmtree(target_dir)

                success, downloaded_file, msg_wget, oss_name, oss_version = download_wget(link, target_dir, compressed_only)
                if success:
                    success = extract_compressed_file(downloaded_file, target_dir, True)
            # Download from rubygems.org
            elif is_rubygems and shutil.which("gem"):
                success = gem_download(link, target_dir, checkout_to)
        if msg:
            msg = f'git fail: {msg}'
            if msg_wget:
                msg = f'{msg}, wget fail: {msg_wget}'
            else:
                msg = f'{msg}, wget success'
    except Exception as error:
        success = False
        msg = str(error)

    logger.info(f"\n* FOSSLight Downloader - Result: {success} ({msg})")
    return success, msg, oss_name, oss_version


def get_ref_to_checkout(checkout_to, ref_list):
    ref_to_checkout = checkout_to
    try:
        checkout_to = checkout_to.strip()
        if checkout_to in ref_list:
            return checkout_to

        for prefix in prefix_refs:
            ref_to_checkout = prefix+checkout_to
            if ref_to_checkout in ref_list:
                return ref_to_checkout

        ref_to_checkout = next(
            x for x in ref_list if x.endswith(checkout_to))
    except Exception as error:
        logger.warning(f"git find ref - failed: {error}")
    return ref_to_checkout


def decide_checkout(checkout_to="", tag="", branch=""):
    if checkout_to:
        ref_to_checkout = checkout_to
    else:
        if branch:
            ref_to_checkout = branch
        else:
            ref_to_checkout = tag
    return ref_to_checkout


def get_github_ossname(link):
    oss_name = ""
    p = re.compile(r'https?:\/\/github.com\/([^\/]+)\/([^\/\.]+)(\.git)?')
    match = p.match(link)
    if match:
        oss_name = f"{match.group(1)}-{match.group(2)}"
    return oss_name


def download_git_clone(git_url, target_dir, checkout_to="", tag="", branch=""):
    ref_to_checkout = decide_checkout(checkout_to, tag, branch)
    msg = ""
    oss_name = get_github_ossname(git_url)
    oss_version = ""

    if platform.system() != "Windows":
        signal.signal(signal.SIGALRM, alarm_handler)
        signal.alarm(SIGNAL_TIMEOUT)
    else:
        alarm = Alarm(SIGNAL_TIMEOUT)
        alarm.start()
    try:
        Path(target_dir).mkdir(parents=True, exist_ok=True)
        repo = git.clone_repository(git_url, target_dir,
                                    bare=False, repository=None,
                                    remote=None, callbacks=None)
        if platform.system() != "Windows":
            signal.alarm(0)
        else:
            del alarm
    except Exception as error:
        logger.warning(f"git clone - failed: {error}")
        msg = str(error)
        return False, msg, oss_name, oss_version
    try:
        if ref_to_checkout != "":
            ref_list = [x for x in repo.references]
            ref_to_checkout = get_ref_to_checkout(ref_to_checkout, ref_list)
            logger.info(f"git checkout: {ref_to_checkout}")
            repo.checkout(ref_to_checkout)

            for prefix_ref in prefix_refs:
                if ref_to_checkout.startswith(prefix_ref):
                    oss_version = ref_to_checkout[len(prefix_ref):]

    except Exception as error:
        logger.warning(f"git checkout to {ref_to_checkout} - failed: {error}")
    return True, msg, oss_name, oss_version


def download_wget(link, target_dir, compressed_only):
    success = False
    msg = ""
    oss_name = ""
    oss_version = ""
    downloaded_file = ""
    if platform.system() != "Windows":
        signal.signal(signal.SIGALRM, alarm_handler)
        signal.alarm(SIGNAL_TIMEOUT)
    else:
        alarm = Alarm(SIGNAL_TIMEOUT)
        alarm.start()
    try:
        Path(target_dir).mkdir(parents=True, exist_ok=True)

        ret, new_link, oss_name, oss_version = get_downloadable_url(link)
        if ret and new_link:
            link = new_link

        if compressed_only:
            for ext in compression_extension:
                if link.endswith(ext):
                    success = True
                    break
        else:
            success = True

        if not success:
            raise Exception('Not supported compression type (link:{0})'.format(link))

        logger.info(f"wget: {link}")
        downloaded_file = wget.download(link, target_dir)
        if platform.system() != "Windows":
            signal.alarm(0)
        else:
            del alarm

        if downloaded_file:
            success = True
            logger.debug(f"wget - downloaded: {downloaded_file}")
    except Exception as error:
        success = False
        msg = str(error)
        logger.warning(f"wget - failed: {error}")

    return success, downloaded_file, msg, oss_name, oss_version


def extract_compressed_dir(src_dir, target_dir, remove_after_extract=True):
    logger.debug(f"Extract Dir: {src_dir}")
    try:
        files_path = [os.path.join(src_dir, x) for x in os.listdir(src_dir)]
        for fname in files_path:
            extract_compressed_file(fname, target_dir, remove_after_extract)
    except Exception as error:
        logger.debug(f"Extract files in dir - failed: {error}")
        return False
    return True


def extract_compressed_file(fname, extract_path, remove_after_extract=True):
    success = True
    try:
        is_compressed_file = True
        if os.path.isfile(fname):
            if fname.endswith(".tar.bz2"):
                decompress_bz2(fname, extract_path)
                os.remove(fname)
                fname = os.path.splitext(fname)[0]

            if fname.endswith(".tar.gz") or fname.endswith(".tgz"):
                with contextlib.closing(tarfile.open(fname, "r:gz")) as t:
                    t.extractall(path=extract_path)
            elif fname.endswith(".tar.xz") or fname.endswith(".tar"):
                with contextlib.closing(tarfile.open(fname, "r:*")) as t:
                    t.extractall(path=extract_path)
            elif fname.endswith(".zip") or fname.endswith(".jar"):
                unzip(fname, extract_path)
            elif fname.endswith(".bz2"):
                decompress_bz2(fname, extract_path)
            else:
                is_compressed_file = False
                success = False
                logger.warning(f"Unsupported file extension: {fname}")

            if remove_after_extract and is_compressed_file:
                logger.debug(f"Remove - extracted file: {fname}")
                os.remove(fname)
        else:
            logger.warning(f"Not a file: {fname}")
    except Exception as error:
        logger.error(f"Extract - failed: {error}")
        success = False
    return success


def decompress_bz2(source_file, dest_path):
    try:
        fzip = bz2.BZ2File(source_file)
        data = fzip.read()  # get the decompressed data
        with open(os.path.splitext(source_file)[0], 'wb') as f:
            f.write(data)  # write a uncompressed file

    except Exception as error:
        logger.error(f"Decompress bz2 - failed: {error}")
        return False
    return True


def unzip(source_file, dest_path):
    try:
        fzip = zipfile.ZipFile(source_file, 'r')
        for filename in fzip.namelist():
            fzip.extract(filename, dest_path)
        fzip.close()
    except Exception as error:
        logger.error(f"Unzip - failed: {error}")
        return False
    return True


def get_gem_version(checkout_to, ver_in_url=""):
    gem_ver = ""
    ver_found = False
    if checkout_to:
        ver_found = True
        gem_ver = checkout_to
    elif ver_in_url:
        ver_found = True
        gem_ver = ver_in_url
    return gem_ver, ver_found


def gem_download(link, target_dir, checkout_to):
    ver_in_url = ""
    success = True
    try:
        # EX) https://rubygems.org/gems/json/versions/2.6.2 -> get 'json'
        gem_name = link.split("rubygems.org/gems/")[-1].split('/')[0]
        # Ex) if version info. is included in url, json/versions/2.6.2 -> get '2.6.2'
        if 'versions/' in link:
            ver_in_url = link.split('versions/')[-1]
        gem_ver, ver_found = get_gem_version(checkout_to, ver_in_url)

        # gem fetch
        if ver_found:
            fetch_cmd = ['gem', 'fetch', gem_name, '-v', gem_ver]
        else:
            fetch_cmd = ['gem', 'fetch', gem_name]
        fetch_result = subprocess.check_output(fetch_cmd, universal_newlines=True)
        fetch_result = fetch_result.replace('\n', '').split(' ')[-1]
        downloaded_gem = f"{fetch_result}.gem"

        # gem unpack
        subprocess.check_output(['gem', 'unpack', downloaded_gem], universal_newlines=True)
        # move unpacked file to target directory
        shutil.move(fetch_result, target_dir)
    except Exception as error:
        success = False
        logger.warning(f"gem download - failed: {error}")
    return success


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import os
from fosslight_util.write_excel import write_result_to_excel, write_result_to_csv, remove_empty_sheet
from fosslight_util.write_opossum import write_opossum
from fosslight_util.write_yaml import write_yaml

SUPPORT_FORMAT = {'excel': '.xlsx', 'csv': '.csv', 'opossum': '.json', 'yaml': '.yaml'}


def check_output_format(output='', format='', customized_format={}):
    success = True
    msg = ''
    output_path = ''
    output_file = ''
    output_extension = ''

    if customized_format:
        support_format = customized_format
    else:
        support_format = SUPPORT_FORMAT

    if format:
        format = format.lower()
        if format not in list(support_format.keys()):
            success = False
            msg = 'Enter the supported format with -f option: ' + ', '.join(list(support_format.keys()))
        else:
            output_extension = support_format[format]

    if success:
        if output != '':
            basename_extension = ''
            if not os.path.isdir(output):
                output_path = os.path.dirname(output)

                basename = os.path.basename(output)
                basename_file, basename_extension = os.path.splitext(basename)
            if basename_extension:
                if format:
                    if output_extension != basename_extension:
                        success = False
                        msg = f"Enter the same extension of output file(-o:'{output}') with format(-f:'{format}')."
                else:
                    if basename_extension not in support_format.values():
                        success = False
                        msg = 'Enter the supported file extension: ' + ', '.join(list(support_format.values()))
                if success:
                    output_file = basename_file
                    output_extension = basename_extension
            else:
                output_path = output

    return success, msg, output_path, output_file, output_extension


def write_output_file(output_file_without_ext, file_extension, sheet_list, extended_header={}, hide_header={}):
    success = True
    msg = ''

    is_not_null, sheet_list = remove_empty_sheet(sheet_list)
    if is_not_null:
        if file_extension == '':
            file_extension = '.xlsx'
        result_file = output_file_without_ext + file_extension

        if file_extension == '.xlsx':
            success, msg = write_result_to_excel(result_file, sheet_list, extended_header, hide_header)
        elif file_extension == '.csv':
            success, msg, result_file = write_result_to_csv(result_file, sheet_list)
        elif file_extension == '.json':
            success, msg = write_opossum(result_file, sheet_list)
        elif file_extension == '.yaml':
            success, msg, result_file = write_yaml(result_file, sheet_list, False)
        else:
            success = False
            msg = f'Not supported file extension({file_extension})'
    else:
        result_file = ""
        msg = "Nothing is detected from the scanner so output file is not generated."

    return success, msg, result_file



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright (c) 2021 LG Electronics
# SPDX-License-Identifier: Apache-2.0
from codecs import open
from setuptools import setup, find_packages

with open('README.md', 'r', 'utf-8') as f:
    readme = f.read()

with open('requirements.txt', 'r', 'utf-8') as f:
    required = f.read().splitlines()

if __name__ == "__main__":
    setup(
        name='fosslight_util',
        version='1.4.39',
        package_dir={"": "src"},
        packages=find_packages(where='src'),
        description='FOSSLight Util',
        long_description=readme,
        long_description_content_type='text/markdown',
        license='Apache-2.0',
        author='LG Electronics',
        url='https://github.com/fosslight/fosslight_util',
        download_url='https://github.com/fosslight/fosslight_util',
        classifiers=['License :: OSI Approved :: Apache Software License',
                     "Programming Language :: Python :: 3",
                     "Programming Language :: Python :: 3.6",
                     "Programming Language :: Python :: 3.7",
                     "Programming Language :: Python :: 3.8",
                     "Programming Language :: Python :: 3.9",
                     "Programming Language :: Python :: 3.10",
                     "Programming Language :: Python :: 3.11", ],
        install_requires=required,
        package_data={'fosslight_util': ['resources/frequentLicenselist.json', 'resources/licenses.json']},
        include_package_data=True,
        extras_require={":python_version<'3.7'": ["pygit2==1.6.1"],
                        ":python_version>='3.7'": ["pygit2>=1.10.1"]},
        entry_points={
            "console_scripts": [
                "fosslight_download = fosslight_util.download:main",
            ]
        }
    )



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2020 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0
import logging
import re
import requests
from npm.bindings import npm_run
from lastversion import latest
from bs4 import BeautifulSoup
from urllib.request import urlopen
import fosslight_util.constant as constant

logger = logging.getLogger(constant.LOGGER_NAME)


def extract_name_version_from_link(link):
    oss_name = ""
    oss_version = ""
    if link.startswith("www."):
        link = link.replace("www.", "https://www.", 1)
    for key, value in constant.PKG_PATTERN.items():
        p = re.compile(value)
        match = p.match(link)
        if match:
            try:
                origin_name = match.group(1)
                if (key == "pypi") or (key == "pypi2"):
                    oss_name = f"pypi:{origin_name}"
                    oss_name = re.sub(r"[-_.]+", "-", oss_name).lower()
                    oss_version = match.group(2)
                elif key == "maven":
                    artifact = match.group(2)
                    oss_name = f"{origin_name}:{artifact}"
                    origin_name = oss_name
                    oss_version = match.group(3)
                elif key == "npm" or key == "npm2":
                    oss_name = f"npm:{origin_name}"
                    oss_version = match.group(2)
                elif key == "pub":
                    oss_name = f"pub:{origin_name}"
                    oss_version = match.group(2)
                elif key == "cocoapods":
                    oss_name = f"cocoapods:{origin_name}"
            except Exception as ex:
                logger.info(f"extract_name_version_from_link {key}:{ex}")
            if oss_name and (not oss_version):
                if key in ["pypi", "maven", "npm", "npm2", "pub"]:
                    oss_version, link = get_latest_package_version(link, key, origin_name)
                    logger.debug(f'Try to download with the latest version:{link}')
            break
    return oss_name, oss_version, link, key


def get_latest_package_version(link, pkg_type, oss_name):
    find_version = ''
    link_with_version = link

    try:
        if pkg_type in ['npm', 'npm2']:
            stderr, stdout = npm_run('view', oss_name, 'version')
            if stdout:
                find_version = stdout.strip()
            link_with_version = f'https://www.npmjs.com/package/{oss_name}/v/{find_version}'
        elif pkg_type == 'pypi':
            find_version = str(latest(oss_name, at='pip', output_format='version', pre_ok=True))
            link_with_version = f'https://pypi.org/project/{oss_name}/{find_version}'
        elif pkg_type == 'maven':
            maven_response = requests.get(f'https://api.deps.dev/v3alpha/systems/maven/packages/{oss_name}')
            if maven_response.status_code == 200:
                find_version = maven_response.json().get('versions')[-1].get('versionKey').get('version')
            oss_name = oss_name.replace(':', '/')
            link_with_version = f'https://mvnrepository.com/artifact/{oss_name}/{find_version}'
        elif pkg_type == 'pub':
            pub_response = requests.get(f'https://pub.dev/api/packages/{oss_name}')
            if pub_response.status_code == 200:
                find_version = pub_response.json().get('latest').get('version')
            link_with_version = f'https://pub.dev/packages/{oss_name}/versions/{find_version}'
    except Exception as e:
        logger.debug(f'Fail to get latest package version({link}:{e})')
    return find_version, link_with_version


def get_downloadable_url(link):

    ret = False
    result_link = link

    oss_name, oss_version, new_link, pkg_type = extract_name_version_from_link(link)
    new_link = new_link.replace('http://', '')
    new_link = new_link.replace('https://', '')

    if pkg_type == "pypi":
        ret, result_link = get_download_location_for_pypi(new_link)
    elif pkg_type == "maven" or new_link.startswith('repo1.maven.org/'):
        ret, result_link = get_download_location_for_maven(new_link)
    elif (pkg_type in ["npm", "npm2"]) or new_link.startswith('registry.npmjs.org/'):
        ret, result_link = get_download_location_for_npm(new_link)
    elif pkg_type == "pub":
        ret, result_link = get_download_location_for_pub(new_link)

    return ret, result_link, oss_name, oss_version


def get_download_location_for_pypi(link):
    # get the url for downloading source file in pypi.org/project/(oss_name)/(oss_version)/#files
    ret = False
    new_link = ''

    try:
        dn_loc_re = re.findall(r'pypi.org\/project\/?([^\/]*)\/?([^\/]*)', link)
        oss_name = dn_loc_re[0][0]
        oss_version = dn_loc_re[0][1]

        pypi_url = 'https://pypi.org/project/' + oss_name + '/' + oss_version + '/#files'

        content = urlopen(pypi_url).read().decode('utf8')
        bs_obj = BeautifulSoup(content, 'html.parser')

        card_file_list = bs_obj.findAll('div', {'class': 'card file__card'})

        for card_file in card_file_list:
            file_code = card_file.find('code').text
            if file_code.lower() == "source":
                new_link = card_file.find('a').attrs['href']
                ret = True
                break
    except Exception as error:
        ret = False
        logger.warning('Cannot find the link for pypi (url:'+link+') '+str(error))

    return ret, new_link


def get_download_location_for_maven(link):
    # get the url for downloading source file in
    # repo1.maven.org/maven2/(group_id(split to separator '/'))/(artifact_id)/(oss_version)
    ret = False
    new_link = ''

    try:
        if link.startswith('mvnrepository.com/artifact/'):
            dn_loc_split = link.replace('mvnrepository.com/', '').split('/')
            group_id = dn_loc_split[1].replace('.', '/')
            dn_loc = 'https://repo1.maven.org/maven2/' + group_id + '/' + dn_loc_split[2] + '/' + dn_loc_split[3]

        elif link.startswith('repo1.maven.org/maven2/'):
            dn_loc_split = link.replace('repo1.maven.org/maven2/', '').split('/')

            if link.endswith('.tar.gz') or link.endswith('.jar') or link.endswith('.tar.xz'):
                new_link = 'https://' + link
                ret = True
                return ret, new_link
            else:
                dn_loc = 'https://' + link
        else:
            raise Exception("not valid url for maven")

        html = urlopen(dn_loc).read().decode('utf8')
        bs_obj = BeautifulSoup(html, 'html.parser')

        file_name = dn_loc.split('/')[-2] + '-' + dn_loc.split('/')[-1] + '-sources.jar'

        for link in bs_obj.findAll("a"):
            if link.text == file_name:
                source_url = link['href']
                new_link = dn_loc + '/' + source_url
                break
            elif link['href'].endswith('sources.jar') or link['href'].endswith('source.jar') or link['href'].endswith('src.jar'):
                source_url = link['href']
                new_link = dn_loc + '/' + source_url

        if new_link != '':
            ret = True

    except Exception as error:
        ret = False
        logger.warning('Cannot find the link for maven (url:'+link+') '+str(error))

    return ret, new_link


def get_download_location_for_npm(link):
    # url format : registry.npmjs.org/packagename/-/packagename-version.tgz
    ret = False
    new_link = ''
    oss_version = ""
    oss_name_npm = ""
    tar_name = ""

    link = link.replace('%40', '@')
    if link.startswith('www.npmjs.com/') or link.startswith('registry.npmjs.org/'):
        try:
            dn_loc_split = link.split('/')
            if dn_loc_split[1] == 'package':
                idx = 2
            else:
                idx = 1
            if dn_loc_split[idx].startswith('@'):
                oss_name_npm = dn_loc_split[idx]+'/'+dn_loc_split[idx+1]
                tar_name = dn_loc_split[idx+1]
                oss_version = dn_loc_split[idx+3]
            else:
                oss_name_npm = dn_loc_split[idx]
                tar_name = oss_name_npm
                oss_version = dn_loc_split[idx+2]

            tar_name = f'{tar_name}-{oss_version}'
            new_link = f'https://registry.npmjs.org/{oss_name_npm}/-/{tar_name}.tgz'
            ret = True
        except Exception as error:
            ret = False
            logger.warning('Cannot find the link for npm (url:'+link+') '+str(error))
    return ret, new_link


def get_download_location_for_pub(link):
    ret = False
    new_link = ''

    # url format : https://pub.dev/packages/(oss_name)/versions/(oss_version)
    # download url format : https://pub.dev/packages/(oss_name)/versions/(oss_version).tar.gz
    try:
        if link.startswith('pub.dev/packages'):
            new_link = f'https://{link}.tar.gz'
            ret = True

    except Exception as error:
        ret = False
        logger.warning('Cannot find the link for pub (url:'+link+') '+str(error))

    return ret, new_link



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2022 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import logging
import os
from fosslight_util.constant import LOGGER_NAME
from fosslight_util.parsing_yaml import parsing_yml

logger = logging.getLogger(LOGGER_NAME)
VERSION = 'version'
LICENSE = 'license'
NAME = 'name'


def compare_yaml(before_file, after_file):
    before_oss_items, _, _ = parsing_yml(before_file, os.path.dirname(before_file))
    after_oss_items, _, _ = parsing_yml(after_file, os.path.dirname(after_file))

    before_items = get_merged_item(before_oss_items)
    after_items = get_merged_item(after_oss_items)

    new_before = []
    for bi in before_items:
        if bi in after_items:
            after_items.remove(bi)
        else:
            new_before.append(bi)

    ADD = "add"
    DELETE = "delete"
    CHANGE = "change"
    PREV = "prev"
    NOW = "now"

    comp_out = {ADD: [], DELETE: [], CHANGE: []}

    b_name = set([bi[NAME] for bi in new_before])
    a_name = set([ai[NAME] for ai in after_items])

    removed_name = b_name - a_name
    changed_name = b_name - removed_name
    added_name = a_name - changed_name

    for bi in new_before:
        if bi[NAME] in removed_name:
            comp_out[DELETE].append(bi)
        elif bi[NAME] in changed_name:
            filtered = next(filter(lambda oss_dict: oss_dict[NAME] == bi[NAME], comp_out[CHANGE]), None)
            if filtered:
                bi.pop(NAME)
                filtered[PREV].append(bi)
            else:
                item_info = {NAME: bi[NAME], PREV: [], NOW: []}
                bi.pop(NAME)
                item_info[PREV].append(bi)
                comp_out[CHANGE].append(item_info)

    for ai in after_items:
        if ai[NAME] in added_name:
            comp_out[ADD].append(ai)
        elif ai[NAME] in changed_name:
            filtered = next(filter(lambda oss_dict: oss_dict[NAME] == ai[NAME], comp_out[CHANGE]), None)
            if filtered:
                ai.pop(NAME)
                filtered[NOW].append(ai)

    return comp_out


def get_merged_item(oss_items):
    item_list = []
    for oi in oss_items:
        if oi.exclude:
            continue
        item_info = {NAME: oi.name, VERSION: oi.version, LICENSE: oi.license}

        filtered = next(filter(lambda oss_dict: oss_dict[NAME] == oi.name and oss_dict[VERSION] == oi.version, item_list), None)
        if filtered:
            filtered[LICENSE].extend(oi.license)
            filtered[LICENSE] = list(set(filtered[LICENSE]))
        else:
            item_list.append(item_info)

    return item_list



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Copyright (c) 2022 LG Electronics Inc.
# SPDX-License-Identifier: Apache-2.0

import os
import uuid
import logging
import re
from pathlib import Path
from spdx.creationinfo import Tool
from spdx.document import Document
from spdx.package import Package
from spdx.relationship import Relationship
from spdx.license import License, LicenseConjunction
from spdx.utils import SPDXNone
from spdx.utils import NoAssert
from spdx.version import Version
from spdx.writers import json
from spdx.writers import yaml
from spdx.writers import xml
from spdx.writers import tagvalue
from fosslight_util.spdx_licenses import get_spdx_licenses_json, get_license_from_nick
import fosslight_util.constant as constant
import traceback

logger = logging.getLogger(constant.LOGGER_NAME)


def get_license_list_version():
    version = 'N/A'
    success, error_msg, licenses = get_spdx_licenses_json()
    if success:
        version = licenses['licenseListVersion']
    else:
        logger.info(f'Fail to get spdx license list version:{error_msg}')
    return version


def write_spdx(output_file_without_ext, output_extension, sheet_list,
               scanner_name, scanner_version, spdx_version=(2, 3)):
    success = True
    error_msg = ''
    if sheet_list:
        doc = Document(version=Version(*spdx_version),
                       data_license=License.from_identifier('CC0-1.0'),
                       namespace=f'http://spdx.org/spdxdocs/{scanner_name.lower()}-{uuid.uuid4()}',
                       name=f'SPDX Document by {scanner_name.upper()}',
                       spdx_id='SPDXRef-DOCUMENT')

        doc.creation_info.set_created_now()
        doc.creation_info.add_creator(Tool(f'{scanner_name.upper()} {scanner_version}'))
        doc.creation_info.license_list_version = Version(*tuple(get_license_list_version().split('.')))

        relation_tree = {}
        spdx_id_packages = []

        output_dir = os.path.dirname(output_file_without_ext)
        Path(output_dir).mkdir(parents=True, exist_ok=True)
        try:
            package_id = 0
            root_package = False
            for sheet_name, sheet_contents in sheet_list.items():
                if sheet_name not in constant.supported_sheet_and_scanner.keys():
                    continue
                scanner = constant.supported_sheet_and_scanner.get(sheet_name)
                for oss_item in sheet_contents:
                    if len(oss_item) < 9:
                        logger.warning(f"sheet list is too short ({len(oss_item)}): {oss_item}")
                        continue
                    package_id += 1
                    package = Package(spdx_id=f'SPDXRef-{package_id}')

                    if oss_item[1] != '':
                        package.name = oss_item[1]  # required
                    else:
                        package.name = SPDXNone()

                    if oss_item[2] != '':
                        package.version = oss_item[2]  # no required

                    if oss_item[4] != '':
                        package.download_location = oss_item[4]  # required
                    else:
                        package.download_location = SPDXNone()

                    if scanner == constant.FL_DEPENDENCY:
                        package.files_analyzed = False  # If omitted, the default value of true is assumed.
                    else:
                        package.files_analyzed = True

                    if oss_item[5] != '':
                        package.homepage = oss_item[5]  # no required

                    if oss_item[6] != '':
                        package.cr_text = oss_item[3]  # required
                    else:
                        package.cr_text = SPDXNone()
                    if oss_item[3] != '':
                        lic_list = [check_input_license_format(lic.strip()) for lic in oss_item[3].split(',')]
                        first_lic = License.from_identifier(lic_list.pop(0))
                        while lic_list:
                            next_lic = License.from_identifier(lic_list.pop(0))
                            license_conjunction = LicenseConjunction(first_lic, next_lic)
                            first_lic = license_conjunction
                        package.license_declared = first_lic
                    else:
                        package.license_declared = NoAssert()  # required

                    doc.add_package(package)

                    if scanner == constant.FL_DEPENDENCY:
                        spdx_id_packages.append([package.name, package.spdx_id])
                        comment = oss_item[8]
                        relation_tree[package.name] = {}
                        relation_tree[package.name]['id'] = package.spdx_id
                        relation_tree[package.name]['dep'] = []

                        if 'root package' in comment.split(','):
                            root_package = True
                            relationship = Relationship(f"{doc.spdx_id} DESCRIBES {package.spdx_id}")
                            doc.add_relationship(relationship)
                        if len(oss_item) > 9:
                            deps = oss_item[9]
                            relation_tree[package.name]['dep'].extend([di.strip().split('(')[0] for di in deps.split(',')])
            if scanner == constant.FL_DEPENDENCY and len(relation_tree) > 0:
                for pkg in relation_tree:
                    if len(relation_tree[pkg]['dep']) > 0:
                        pkg_spdx_id = relation_tree[pkg]['id']
                        if len(relation_tree[pkg]['dep']) > 0:
                            for pname in relation_tree[pkg]['dep']:
                                ans = next(filter(lambda x: x[0] == pname, spdx_id_packages), None)
                                if ans is None:
                                    continue
                                rel_pkg_spdx_id = ans[1]
                                relationship = Relationship(f'{pkg_spdx_id} DEPENDS_ON {rel_pkg_spdx_id}')
                                doc.add_relationship(relationship)
                if not root_package:
                    root_package = Package(spdx_id='SPDXRef-ROOT-PACKAGE')
                    root_package.name = 'root package'
                    root_package.download_location = NoAssert()
                    root_package.files_analyzed = False
                    root_package.cr_text = SPDXNone()
                    root_package.license_declared = NoAssert()
                    doc.add_package(root_package)
                    relationship = Relationship(f"{doc.spdx_id} DESCRIBES {root_package.spdx_id}")
                    doc.add_relationship(relationship)
        except Exception as e:
            success = False
            error_msg = f'Failed to create spdx document object:{e}, {traceback.format_exc()}'
    else:
        success = False
        error_msg = 'No item to write in output file.'

    result_file = ''
    if success:
        result_file = output_file_without_ext + output_extension
        try:
            out_mode = "w"
            if result_file.endswith(".tag"):
                writer_module = tagvalue
            elif result_file.endswith(".json"):
                writer_module = json
            elif result_file.endswith(".xml"):
                writer_module = xml
            elif result_file.endswith(".yaml"):
                writer_module = yaml
            else:
                raise Exception("FileType Not Supported")

            with open(result_file, out_mode) as out:
                writer_module.write_document(doc, out, True)
        except Exception as e:
            success = False
            error_msg = f'Failed to write spdx document: {e}'
            if os.path.exists(result_file):
                os.remove(result_file)

    return success, error_msg, result_file


def convert_to_spdx_style(input_string):
    input_string = re.sub(r'[^\w\s\.\-]', '', input_string)
    input_string = re.sub(r'[\s\_]', '-', input_string)
    input_converted = f"LicenseRef-{input_string}"
    return input_converted


def check_input_license_format(input_license):
    spdx_licenses = get_spdx_licensename()
    for spdx in spdx_licenses:
        if input_license.casefold() == spdx.casefold():
            return spdx

    if input_license.startswith('LicenseRef-'):
        return input_license

    licensesfromJson = get_license_from_nick()
    if licensesfromJson == "":
        logger.warning(" Error - Return Value to get license from Json is none")

    try:
        converted_license = licensesfromJson.get(input_license.casefold())
        if converted_license is None:
            converted_license = convert_to_spdx_style(input_license)
    except Exception as ex:
        logger.warning(f"Error - Get frequetly used license : {ex}")

    return converted_license


def get_spdx_licensename():
    spdx_licenses = []
    try:
        success, error_msg, licenses = get_spdx_licenses_json()
        if success is False:
            logger.warning(f"Error to get SPDX Licesens : {error_msg}")

        licenseInfo = licenses.get("licenses")
        for info in licenseInfo:
            shortID = info.get("licenseId")
            isDeprecated = info.get("isDeprecatedLicenseId")
            if isDeprecated is False:
                spdx_licenses.append(shortID)
    except Exception as ex:
        logger.warning(f"Error access to get_spdx_licenses_json : {ex}")
    return spdx_licenses
